# -*- coding: ISO-8859-1 -*-
""" capellaScript -- Copyright (c) 2005 Peter Becker
>>> ClubOrgel_A : Tabulaturschrift fr Club-Handorgel
    Umsetzen von Noten in die Griffschrift fr die Club-Handorgel.
    -> Schritt 1 von 3||
    Umwandeln in Zieltonart C/F/B.|
    oder ohne Transposition aber ohne Bsse.|
    Diese Zieltonart ist nur fr den Schritt 2 relevant,|
    da dieser Schritt von der C/F/B Stimmung ausgeht.|
    Die entstehende Griffschrift passt aber auch zu|
    jeder anderen Stimmung||
    Bei Problemen bitte eine Mail an peter_becker@freenet.de
<<<

History:  01.07.05 - Erste Ausgabe
          13.07.05 - Akkorde auch an Pausen
          22.02.06 - Auswahl "as is" - ohne Bsse, keine Transposition
          06.04.06 - Balgrichtung falls keine Transposition
          23.04.06 - Support fr bersetzen ohne Transposition
          28.04.06 - Pseudo Akkorde Z und R fr 'ohne Transposition'
          10.05.06 - Menu Fix bei 3 Tonarten nebeneinander
          24.05.06 - Bugfix - normale Textobjekte gehen bei Transposition nicht mehr verloren
                     Untersttzung von R und Z bei Transposition
          14.05.10 - Fehler bei zu groem Tonartenabstand beseitigt
          12.12.10 - Nachrichten im Men bereinigt
          16.01.11 - berprfung ob Partitur den geforderten Vorgaben entspricht
          08.05.12 - V5.1.6 wg ClubOrgel_C
          13.07.12 - V5.1.7 F-Dur Dominant Akkord auf Zug gendert ( Tippfehler )
          28.12.12   V5.2.0 Instrumententyp whlbar
          08.12.13   V5.2.1 wg. ClubOrgel_B
          31.08.16   V5.2.2 Schriftgre Begleitung eingefhrt
          04.10.16   V5.2.3 wg. Module C
          18.12.17   V5.2.4 wg. Module C
          18.12.17   V5.2.5 wg. Module C
          13.06.20   V5.2.6 wg. Module B

"""

import xml.dom, tempfile
from caplib.capDOM import ScoreChange
from xml.dom.minidom import NodeList, Node, Element

checkCapVersion(5,1,5)
doc = [] # parentNode von score
version = '5.2.6'

def defineDir():                                                      # Balgrichtung bei nicht untersttzten Tonarten vorgeben
                                                                      # Hier werden nur die Pseudo Akkorde R und Z abgefragt
    global tArt, tList, tBgl

    #messageBox('DefDir','tArt=' + str(tArt) + '\n' + 'tList=' + str(tList) + '\n' + '\n' + 'tBgl=' + str(tBgl)) 

    DirList  = [ 'R' ]
    valChord = ['R','Z' ]                                             # R=Druck , Z=Zug

    for no in activeScore().noteObjs():
        if no.isChord()or no.isRest():
            i = no.nDrawObjs()-1
            while i >= 0:
                d = no.drawObj(i)
                if d['type'] == 'transposable':                       # transponierbare Objekte lschen
                    ix = i                                            # Index zum Akkord merken
                    i -= 1
                    no.deleteDrawObj(ix)
                    continue
                elif d['type'] == 'text':                             # Text Objekte
                    txt = d['content']
                    if txt in valChord:                               # prfen ob gltiger Akkord
                        ix  = i                                       # Index zum Akkord merken 
                        i -= 1                       
                    else:
                        i -= 1
                        continue
                else:
                    i -= 1
                    continue
                
                t = dict(type = 'text', x = 0, y = -3.0,
                    font = dict(height = 12, pitchAndFamily = 0, face = 'Times New Roman', weight = 0))

                if txt == 'D#': txt = 'Dis'                           # Schreibweise anpassen wegen Phyton Kommentarzeichen
                
                if txt in DirList and d['type'] == 'text':            # Balgrichtung prfen
                    t['content'] = str(' ')                           # Textfeld leeren 
                    t['tag'] = '2086-32'                              # auf Druck spielen
                elif txt not in DirList and d['type'] == 'text':
                    t['content'] = str(' ')                           # Textfeld leeren
                    t['tag'] = '2086-42'                              # auf Zug spielen
                no.replaceDrawObj(ix,t)                               # und eintragen
    return
                                
def transposeAcc():             # Akkordbezeichnungen nach C, F oder Bb-Dur konvertieren
                                
    global tArt, tList, okeyw
    # Dur Akkorde der einzelnen Tonarten
    #messageBox('Interface','tArt = ' + str(tArt) + '\ntList = ' + str(tList))
    TransposeList  = dict(
                      P0 = dict(T='C'  , S='F' , D='G' , D7='G7'  , DD='D7' , M2='Dm'  , M6='Am') ,
                      P1 = dict(T='G'  , S='C' , D='D' , D7='D7'  , DD='A7' , M2='Am'  , M6='Em') ,
                      P2 = dict(T='D'  , S='G' , D='A' , D7='A7'  , DD='E7' , M2='Em'  , M6='Bm') ,
                      P3 = dict(T='A'  , S='D' , D='E' , D7='E7'  , DD='B7' , M2='Bm'  , M6='F#m'),
                      P4 = dict(T='E'  , S='A' , D='B' , D7='B7'  , DD='F#7', M2='F#m' , M6='C#m'),
                      P5 = dict(T='B'  , S='E' , D='F#', D7='F#7' , DD='C#7', M2='C#m' , M6='G#m'),
                      P6 = dict(T='F#' , S='B' , D='C#', D7='C#7' , DD='G#7', M2='G#m' , M6='D#m'),
                      P7 = dict(T='C#' , S='F#', D='G#', D7='G#7' , DD='D#7', M2='D#m' , M6='A#m'),
                      M1 = dict(T='F'  , S='Bb', D='C' , D7='C7'  , DD='G7' , M2='Gm'  , M6='Dm') ,
                      M2 = dict(T='Bb' , S='Eb', D='F' , D7='F7'  , DD='C7' , M2='Cm'  , M6='Gm') ,
                      M3 = dict(T='Eb' , S='Ab', D='Bb', D7='Bb7' , DD='F7' , M2='Fm'  , M6='Cm') ,
                      M4 = dict(T='Ab' , S='Db', D='Eb', D7='Eb7' , DD='Bb7', M2='Bbm' , M6='Fm') ,
                      M5 = dict(T='Db' , S='Gb', D='Ab', D7='Ab7' , DD='Eb7', M2='Ebm' , M6='Bbm'),
                      M6 = dict(T='Gb' , S='Cb', D='Db', D7='Db7' , DD='Ab7', M2='Abm' , M6='Ebm'),
                      M7 = dict(T='Cb' , S='Fb', D='Gb', D7='Gb7' , DD='Db7', M2='Dbm' , M6='Abm')
                      )

    #xx = transposeList['M1']['T']  #so wird das Dictionary abgefragt

    valChord = ['C','D','E','F','G','A','B','H','C#','D#','E#','F#','G#','A#','B#','H#','Cis','Dis','Eis','Fis',
            'Gis','Ais','Bis','His','Cb','Db','Eb','Fb','Gb','Ab','Bb','Ces','Des','Es','Fes','Ges','As',
            'Cm','Dm','Em','Fm','Gm','Am','Bm','Hm','C#m','D#m','E#m','F#m','G#m','A#m','B#m','H#m','Cism',
            'Dism','Eism','Fism','Gism','Aism','Bism','Hism','Cbm','Dbm','Ebm','Fbm','Gbm','Abm','Bbm',
            'Cesm','Desm','Esm','Fesm','Gesm','Asm','C7','D7','E7','F7','G7','A7','B7','H7','C#7','D#7',
            'E#7','F#7','G#7','A#7','B#7','H#7','Cis7','Dis7','Eis7','Fis7','Gis7','Ais7','Bis7','His7',
            'Cb7','Db7','Eb7','Fb7','Gb7','Ab7','Bb7','Ces7','Des7','Es7','Fes7','Ges7','As7','R','Z' ]

    
    t = dict(type = 'text', x = 0, y = -3.0,
        font = dict(height = 12, pitchAndFamily = 0, face = 'Times New Roman', weight = 0))
    
    type      = 'text'
    for no in activeScore().noteObjs():
        if no.isChord()or no.isRest():
            i = no.nDrawObjs()-1
            while i >= 0:
                d = no.drawObj(i)
                if d['type'] == 'transposable':
                    type = 'trans'
                    break
                else:
                    i -= 1
            if type == 'trans':
                break
            
    for no in activeScore().noteObjs():
        if no.isChord()or no.isRest():
            keyw  = str(no.curKey())
            okeyw = keyw
            if keyw[0:1] == '-':
               keyw = 'M' + str(keyw[1])
            else :
                keyw = 'P' + str(keyw)
 
            i = no.nDrawObjs()-1
            while i >= 0:
                d = no.drawObj(i)
                #messageBox('TRANS',str(d))
                if d['type'] == 'transposable' and type == 'trans':
                    nRef = d['nRefNote']
                    relN = RelDiatonicNote.fromCircleOfFifth(nRef)
                    txt = str(relN)
                    #messageBox('TRANS',str(nRef) + '    ' + str(txt))
                    rc = isMoll(no,txt)           # prfen ob Moll
                    #messageBox('Moll',str(rc))
                    if rc == 1:                   # wenn ja,
                        txt = txt + 'm'           # dann Akkord bernehmen
                    elif rc == 2:
                        txt = txt + '7'
                        
                    #messageBox('TRANS1',str(nRef) + '    ' + str(txt))
                    
                elif d['type'] == 'text' and type == 'text':
                    txt = d['content']
                else:
                    i -= 1
                    continue
                
                if txt not in valChord:                                           # normale Textobjekte bleiben erhalten
                    #messageBox('noChordText',str(txt))
                    i -= 1
                    continue
                
                #messageBox('ChordText',str(txt))                                 # liefert die Akkordstufen der aktuellen Tonart

                if  txt == 'R' or txt == 'Z':                                     # Untersttzung fr Dummy Akkorde bei Transposition
                    if txt == 'R':
                        t['content'] = 'R'
                        t['tag'] = '2086-32' # Druck spielen 
                    else:
                        t['content'] = 'Z'
                        t['tag'] = '2086-42' # Zug spielen  
                    no.replaceDrawObj(i,t)
                    i -= 1
                    continue

                ton = TransposeList[keyw]['T']    # Tonika
                sub = TransposeList[keyw]['S']    # Subdominante 
                dom = TransposeList[keyw]['D']    # Dominante
                do7 = TransposeList[keyw]['D7']   # Dominantsept
                dod = TransposeList[keyw]['DD']   # Doppeldominante
                m2  = TransposeList[keyw]['M2']   # Mollakkord der 2. Stufe
                m6  = TransposeList[keyw]['M6']   # Mollakkord der 6. Stufe
                
                if tArt == 'C' and len(tList) == 3:        # 3 Tonarten nebeneinander
                    if str(tList[0]) == str(okeyw):
                        wo = 'B'
                    elif str(tList[1]) == str(okeyw):
                        wo = 'F'
                    elif str(tList[2]) == str(okeyw):
                        wo = 'C'
                elif tArt == 'F' and len(tList) == 2:      # 2 Tonarten
                    if tList[1] - tList[0] == 1:           # 2 Tonarten nebeneinander
                        if str(tList[0]) == str(okeyw):
                            wo = 'F'
                        elif str(tList[1]) == str(okeyw):
                            wo = 'C'
                    else:
                        if str(tList[0]) == str(okeyw):    # 2 Tonarten nicht nebeneinander
                            wo = 'B'
                        elif str(tList[1]) == str(okeyw):
                            wo = 'C'
                elif tArt == 'C' and len(tList) == 2:      # 2 Tonarten
                    if tList[1] - tList[0] == 1:           # 2 Tonarten nebeneinander
                        if str(tList[0]) == str(okeyw):
                            wo = 'B'
                        elif str(tList[1]) == str(okeyw):
                            wo = 'F'
                    else:
                        if str(tList[0]) == str(okeyw):    # 2 Tonarten nicht nebeneinander
                            wo = 'B'
                        elif str(tList[1]) == str(okeyw):
                            wo = 'C'

                else: wo = tArt                            # 1 Tonart, Lage vorgewhlt        


                #messageBox('KEY','txt = ' + str(txt) + '  wo = ' + str(wo) + '\n' + str(keyw) + ' : ' + str(ton) + str(sub) + str(dom))
                mtx = txt
                txt = txt[0:1]
                ton = ton[0:1]
                sub = sub[0:1]
                dom = dom
                do7 = do7
                dod = dod
                m2  = m2
                m6  = m6
                done = 0
                #messageBox('KEY','txt = ' + str(txt) + '\nmtx = ' + str(mtx) + '\nwo = ' + str(wo) + '\nton = ' + str(ton) + '\nsub = ' + str(sub) + '\ndom = ' + str(dom) + '\ndo7 = ' + str(do7) + '\ndod = ' + str(dod) + '\nm2 = ' + str(m2))
                # tag : 2086-aa    a: 31=Druck 41=Zug 
                if txt == ton:
                    if wo == 'C':
                        t['content'] = 'C'
                        t['tag'] = '2086-31' # auf Druck spielen   
                    elif  wo == 'F':
                        t['content'] = 'F'
                        t['tag'] = '2086-31' # auf Druck spielen
                    elif  wo == 'B':
                        t['content'] = 'Bb'
                        t['tag'] = '2086-41' # auf Zug spielen
                    if mtx[1:2] == '7':      # falls Sept obwohl Tonika, die 7 dranhngen
                        t['content'] = t['content'] + '7'
                    
                elif txt == sub:
                    if wo == 'C':
                        t['content'] = 'F'   
                        t['tag'] = '2086-31' # auf Druck spielen
                        done = 1
                    elif  wo == 'F':
                        t['content'] = 'Bb'
                        t['tag'] = '2086-41' # auf Zug spielen
                        done = 1
                    elif  wo == 'B':
                        t['content'] = 'Eb'
                        t['tag'] = '2086-31' # auf Druck spielen
                        done = 1
                    
                elif mtx == do7:
                    if wo == 'C':
                        t['content'] = 'G7'
                        t['tag'] = '2086-41' # auf Zug spielen
                        done = 1
                    elif  wo == 'F':
                        t['content'] = 'C7'
                        t['tag'] = '2086-41' # auf Zug spielen
                        done = 1
                    elif  wo == 'B':
                        t['content'] = 'F7'
                        t['tag'] = '2086-31' # auf Druck spielen
                        done = 1
                    
                elif mtx == dom:
                    if wo == 'C':
                        t['content'] = 'G'
                        t['tag'] = '2086-41' # auf Zug spielen
                        done = 1
                    elif  wo == 'F':
                        t['content'] = 'C'
                        t['tag'] = '2086-41' # auf Zug spielen
                        done = 1
                    elif  wo == 'B':
                        t['content'] = 'F'
                        t['tag'] = '2086-31' # auf Druck spielen
                        done = 1
                    
                elif txt == dod:
                    if wo == 'C':
                        t['content'] = 'D0'
                        t['tag'] = '2086-41' # auf Zug spielen nur Bass kein Akkord
                        done = 1
                    elif  wo == 'F':
                        t['content'] = 'G7'
                        t['tag'] = '2086-41' # auf Zug spielen
                        done = 1
                    elif  wo == 'B':
                        t['content'] = 'C7'
                        t['tag'] = '2086-41' # auf Zug spielen
                        done = 1

                if mtx == m2:
                    if wo == 'C':
                        t['content'] = 'Dm'
                        t['tag'] = '2086-41' # auf Zug spielen
                    elif  wo == 'F':
                        t['content'] = 'Bb'
                        t['tag'] = '2086-41' # auf Zug spielen Gm durch Bb-Dur ersetzen
                    elif  wo == 'B':
                        t['content'] = 'Eb'
                        t['tag'] = '2086-31' # auf Druck spielen Cm durch Eb-Dur ersetzen
                        done = 1
                    
                if mtx == m6:
                    if wo == 'C':
                        t['content'] = 'A0'
                        t['tag'] = '2086-31' # auf Druck spielen nur Bass
                        done = 1
                    elif  wo == 'F':
                        t['content'] = 'Dm'
                        t['tag'] = '2086-41' # auf Zug spielen
                        done = 1
                    elif  wo == 'B':
                        t['content'] = 'G0'
                        t['tag'] = '2086-41' # auf Zug spielen nur Bass
                        done = 1
                    
                no.replaceDrawObj(i,t)
                i -= 1
    return        

def isMoll(note,txt):                        # prfen ob bei einem transponierbaren Objekt ein Mollakkord vorliegt
                                             # oder um einen Septimakkord handelt
                                             # rx 1=Moll, rc=2 Septime
    #for note in activeScore().noteObjs():
    a   = 0
    acc = txt
    #messageBox('Moll',str(note) + '\n' + str(acc))
    for d in note.drawObjs():
        if d['type'] == 'transposable':
            liste = d['items']
            #messageBox('Liste',str(liste))
            liste.reverse()
            lastElement = liste.pop()
            #messageBox('LastElement',str(lastElement))
            if 'content' in lastElement:                         # Fuchs 1
                ak = lastElement['content']                      # Akkordbezeichnung
                #messageBox('Content1a',str(ak))
                if ak == 'Transponierbare Symbole':
                    lastElement = liste.pop()
                    akkord = lastElement['data']
                    #messageBox('Content1b',str(akkord))
                    ix = akkord.find('plain')
                    ak = akkord[ix:]
                    #messageBox('Content1c',str(ak))             # Fuchs 3, Capella Standard 2 ( C7 Symbol )
                ak1 = ak[0]
                if ak1 in ['c','d','e','f','g','a','h','b']:     # Capella Standard 1  ( gelieferte transp. Sym. )
                    a = 1
                    break   
                if 'm' in ak:
                    a = 1                                        # wenn m drin, dann Moll
                    break
                
            elif 'items' in lastElement:                         # Fuchs 2 und Capella Accordsymbole.py
                #messageBox('items',str(lastElement))
                liste.reverse()
                lastElement = liste.pop()
                typ = lastElement['type']
                if typ == 'group':                                
                    liste = lastElement['items']
                    ak = liste.pop()                              # m = Moll
                    ac = ak['content']
                    #messageBox('items 1',str(ac))
                    if 'm' in str(ac):
                        a = 1                                     # wenn m drin, dann Moll
                        break
                    elif '7' in str(ac):
                        a = 2                                     # wenn 7 drin, dann Septime
                        break
                else:
                    messageBox('Element : items','nicht untersttztes Element : \n' + str(lastElement))                    

            elif 'data' in lastElement:                           # Fuchs 4 - Septime hochgestellt
                #messageBox('data',str(lastElement))              
                ak = lastElement['data']
                ix = ak.find('plain')
                ac = ak[ix:]
                iy = ak.find('fs21 m')
                ad = ak[iy:]
                iz = ak.find('fs18 7')
                ae = ak[iz:]
                iw = ak.find('fs21 7')
                af = ak[iw:]
                #messageBox('fffff',str(ae)+ 'n' + str(iz))
                if 'm' in ac:
                    a = 1                                         # wenn m drin, dann Moll
                    break
                elif 'fs21 m' in ad:                              # Capella Scan Moll
                    a = 1
                    #messageBox('fffff',str(ak))
                    break
                elif 'fs18 7' in ae or 'fs21 7' in af:            # Capella Scan Septime
                    a = 2
                    #messageBox('fffff',str(ak))
                    break
            else:                                                 # 
                
                messageBox('Element : items','nicht untersttztes Element : \n' + str(lastElement))
                break
        
            break      

    #messageBox('Return',str(a))
    return a                                                          # enthlt der String ein m, dann Moll


def transposeNeu(transposeFrom, transposeTo):

    transposeFrom = int(transposeFrom)
    transposeTo   = int(transposeTo)    
    
    toC = {7:[-1,-1],   # cis
           6:[-4,0],    # fis
           5:[-7,0],    # h
           4:[-3,0],    # e
           3:[-6,0],    # a
           2:[-2,0],    # d
           1:[-5,0],    # g
           0:[0,-1],    # c
           -1:[-3,-1],  # f
           -2:[-6,-1],  # b
           -3:[-2,-1],  # es
           -4:[-5,-1],  # as
           -5:[-1,-1],  # des
           -6:[-4,-1],  # ges
           -7:[-7,0]}   # ces

    fromC = {7:[-7,0],  # cis
             6:[-4,0],  # fis
             5:[-1,-1], # h
            4:[3,0],    # e
            3:[6,0],    # a
            2:[2,0],    # d
            1:[5,0],    # g
            0:[0,-1],   # c
           -1:[3,-1],   # f
           -2:[6,-1],   # b
           -3:[2,-1],   # es
           -4:[5,-1],   # as
           -5:[1,-1],   # des
           -6:[4,-1],   # ges
           -7:[7,0]}    # ces

    #messageBox('Interface','transposeFrom = ' + str(transposeFrom) + '\ntransposeTo = ' + str(transposeTo))
    for system in activeScore().systems():
        for staff in system.staves(): 
            staff.transpose(toC[transposeFrom])
            staff.transpose(fromC[transposeTo])
            if transposeTo > 5:
                staff.transpose([7,-1])  #  + 1 Oktave
    return
            
def transposeKey(tArt, tList):
    #global tArt, tList                             
    #messageBox('XX',str(tArt) + '\n' + str(tList))
    global okeyw
    for no in activeScore().noteObjs():
        if no.isChord():
            keyw  = str(no.curKey())
            okeyw = keyw
            #messageBox('KEY vor Analyse','Tonart = ' + str(keyw) + '\ntArt = ' + str(tArt))
            transposeFrom = okeyw 
            if keyw[0:1] == '-':
                keyw = 'M' + str(keyw[1])
            else :
                keyw = 'P' + str(keyw)
                
            if len(tList) == 1:                        # nur eine Tonart 
                if tArt == 'B':
                    transposeTo = -2
                elif tArt == 'F':
                    transposeTo = -1
                elif tArt == 'C':
                    transposeTo = 0

            elif len(tList) == 2:                      # 2 Tonarten
                #messageBox('XX',str(tArt) + '\n' + str(tList) + '\n' + str(okeyw))
                if tArt == 'F':                        # C/F
                    if str(tList[0]) == str(okeyw):
                        transposeTo = -1
                    elif str(tList[1]) == str(okeyw):
                        transposeTo = 0
                elif tArt == 'C':                      # F/B
                    if str(tList[0]) == str(okeyw):
                        transposeTo = -2
                    elif str(tList[1]) == str(okeyw):
                        transposeTo = -1
                    else:                              # C/B
                        if str(tList[0]) == str(okeyw):
                            transposeTo = -2
                        elif str(tList[1]) == str(okeyw):
                            transposeTo = 0
                
            elif tArt == 'C' and len(tList) == 3:      # 3 Tonarten nebeneinander
                if str(tList[0]) == str(okeyw):
                    transposeTo = -2
                elif str(tList[1]) == str(okeyw):
                    transposeTo = -1
                elif str(tList[2]) == str(okeyw):
                    transposeTo = 0
    
            break

    transposeNeu(transposeFrom, transposeTo)
    return                                                   

def transDialog1():
    global tArt, tBgl
    
    placeholder    = Label('                                            ',width=1)
    lablKey1       = Label('Wie soll die Partitur zum bersetzen vorbereitet werden ?',width=6)
    lablKey2       = Label('            ',width=6)    
    radDefBgl      = Radio(['mit Begleitstimme','ohne Begleitstimme - nur Pseudoakkorde werden bercksichtigt!'],text='Begleitstimme',value=0)
    radDefKey      = Radio(['C-Dur','F-Dur','Bb-Dur','andere Tonart beibehalten ( = ohne Begleitstimme )'],text='Tonart',value=0)
    subboxDefBgl   = HBox([radDefBgl,placeholder],padding=4)
    subboxDefKey   = HBox([radDefKey,placeholder],padding=4)
    box  = VBox([lablKey1,lablKey2,subboxDefBgl,subboxDefKey,placeholder],padding=1)
    dlg  = Dialog('Club_Orgel_A :     Version : ' + version, box)
    if dlg.run():
        if radDefBgl.value() == 0:
            tBgl = 'M'
        elif radDefBgl.value() == 1:
            tBgl = 'N'
        if radDefKey.value() == 0:
            tArt = 'C'
        elif radDefKey.value() == 1:
            tArt = 'F'
        elif radDefKey.value() == 2:
            tArt = 'B'
        elif radDefKey.value() == 3:
            tArt = 'N'
            tBgl = 'N'
        return True
    else:
        return False


def transDialog2():
    global tArt, tBgl
    
    placeholder    = Label('                                            ',width=1)
    lablKey1       = Label('Wie soll die Partitur zum bersetzen vorbereitet werden ?',width=6)
    lablKey2       = Label('            ',width=6)
    radDefBgl      = Radio(['mit Begleitstimme','ohne Begleitstimme - nur Pseudoakkorde werden bercksichtigt!'],text='Begleitstimme',value=0)
    radDefKey      = Radio(['C-Dur/F-Dur','F-Dur/Bb-Dur','andere Tonart beibehalten ( = ohne Begleitstimme )'],text='Tonart',value=0)
    subboxDefBgl   = HBox([radDefBgl,placeholder],padding=4)
    subboxDefKey   = HBox([radDefKey,placeholder],padding=4)
    box  = VBox([lablKey1,lablKey2,subboxDefBgl,subboxDefKey,placeholder],padding=1)
    dlg  = Dialog('Club_Orgel_A :     Version : ' + version, box)
    if dlg.run():
        if radDefBgl.value() == 0:
            tBgl = 'M'
        elif radDefBgl.value() == 1:
            tBgl = 'N'
        if radDefKey.value() == 0:
            tArt = 'F'
        elif radDefKey.value() == 1:
            tArt = 'C'
        elif radDefKey.value() == 2: 
            tArt = 'N'
            tBgl = 'N'    
        return True
    else:
        return False


def transDialog3():
    global tArt, tBgl
    
    placeholder    = Label('                                            ',width=1)
    lablKey1       = Label('Wie soll die Partitur zum bersetzen vorbereitet werden ?',width=6)
    lablKey2       = Label('            ',width=6)
    radDefBgl      = Radio(['mit Begleitstimme','ohne Begleitstimme - nur Pseudoakkorde werden bercksichtigt!'],text='Begleitstimme',value=0)
    radDefKey      = Radio(['C-Dur/F-Dur/Bb-Dur','andere Tonart beibehalten ( = ohne Begleitstimme )'],text='Tonart',value=0)
    subboxDefBgl   = HBox([radDefBgl,placeholder],padding=4)
    subboxDefKey   = HBox([radDefKey,placeholder],padding=4)
    box  = VBox([lablKey1,lablKey2,subboxDefBgl,subboxDefKey,placeholder],padding=1)
    dlg  = Dialog('Club_Orgel_A :     Version : ' + version, box)
    if dlg.run():
        if radDefBgl.value() == 0:
            tBgl = 'M'
        elif radDefBgl.value() == 1:
            tBgl = 'N'
        if radDefKey.value() == 0:
            tArt = 'C'
        elif radDefKey.value() == 1: 
            tArt = 'N'
            tBgl = 'N'    
        return True
    else:
        return False
                                              
# Hauptprogramm:

from caplib.capDOM import ScoreChange
import tempfile

global tArt, tBgl, okeyw

tList    = []
tArt     = 'C'
okeyw    = ' '
activeScore().registerUndo("ClubOrgel_A")

AnzahlZeilen = 0                                                # Partitur berprfen ob die Vorgaben eingehalten sind
AnzahlStimmen = 0
for system in activeScore().systems():
    if system.nStaves() > 1:
        AnzahlZeilen = AnzahlZeilen + 1
        st = 0
        for staff in system.staves():
            if st <> 0:
                for voice in staff.voices():
                    for note in voice.noteObjs():
                        note.setColor(Color.RGB(255,0,0))
            st = 1        
    for staff in system.staves():
        if staff.nVoices() > 1:
            AnzahlStimmen = AnzahlStimmen + 1
            vo = 0
            for voice in staff.voices():
                if vo <> 0:
                    for note in voice.noteObjs():
                        note.setColor(Color.RGB(255,0,0))
                vo = 1
if AnzahlZeilen <> 0 and AnzahlStimmen <> 0:
    fehler ='Die Partitur entspricht nicht den Vorgaben. \n-Mehr als eine Zeile pro System\n-Mehr als eine Stimme pro Zeile\n\nDiese Noten werden nicht bersetzt.'
elif AnzahlZeilen <> 0 and AnzahlStimmen == 0:
    fehler ='Die Partitur entspricht nicht den Vorgaben. \n-Mehr als eine Zeile pro System\n\nDiese Noten werden nicht bersetzt.'
elif AnzahlZeilen == 0 and AnzahlStimmen <> 1:
    fehler ='Die Partitur entspricht nicht den Vorgaben. \n-Mehr als eine Stimme pro Zeile\n\nDiese Noten werden nicht bersetzt.'
if AnzahlZeilen <> 0 or AnzahlStimmen <> 0:
    messageBox('Partiturfehler',str(fehler))


for no in activeScore().noteObjs():                  # Tonarten festhalten
    if no.isChord():
        tonart = no.curKey()
        if tonart not in tList:
            tList.append(tonart)
tList.sort()

tBgl = 'Y'
flag  = 'OK'
delta = tList[len(tList)-1] - tList[0]

if len(tList) > 3:
    flag = 'exit'
elif tList[len(tList)-1] - tList[0] > 2:
    flag = 'exit'

#messageBox('tBgl',str(tList) + '\n' + str(flag) + '\n' + str(delta))

if len(tList) == 1 and flag == 'OK':                  # nur eine Tonart
    if transDialog1():                                # gezielt transponieren
        if tArt <> 'N':  
            transposeAcc()                            # Akkorde transponieren
            transposeKey(tArt, tList)                 # Tonart normalisieren
        else: defineDir()                             # bei nicht untersttzten Tonarten Balg Richtung festlegen
        
elif len(tList) == 2 and flag == 'OK' and delta == 1: # 2 Tonarten nebeneinander
    if transDialog2():                                # gezielt transponieren
        if tArt <> 'N':  
            transposeAcc()                            # Akkorde transponieren
            transposeKey(tArt, tList)                 # Tonart normalisieren
        else: defineDir()                             # bei nicht untersttzten Tonarten Balg Richtung festlegen

elif len(tList) == 2 and flag == 'OK' and delta == 2: # 2 Tonarten mit einer Tonart Abstand 
    if transDialog3():                                # gezielt transponieren
        if tArt <> 'N':  
            transposeAcc()                            # Akkorde transponieren
            transposeKey(tArt, tList)                 # Tonart normalisieren
        else: defineDir()                             # bei nicht untersttzten Tonarten Balg Richtung festlegen

elif len(tList) == 3 and flag == 'OK' and delta == 2: # 3 Tonarten nebeneinander
    if transDialog3():                                # gezielt transponieren
        if tArt <> 'N':  
            transposeAcc()                            # Akkorde transponieren
            transposeKey(tArt, tList)                 # Tonart normalisieren
        else: defineDir()                             # bei nicht untersttzten Tonarten Balg Richtung festlegen
        
else:
    if flag == 'OK':                                  # 2 Tonarten mit einer Abstand
        if tArt <> 'N':  
            tArt = 'C'                                # Alles Transponieren
            transposeAcc()                            # Akkorde transponieren
            transposeKey(tArt, tList)                 # Tonart normalisieren
        else: defineDir()                             # bei nicht untersttzten Tonarten Balg Richtung festlegen
        
if tBgl <> 'N':
    if flag == 'exit': messageBox('Input Error','Umwandlung nicht mglich.\nZu viele Tonarten oder Abstand zwischen den Tonarten zu gro')        
else:
    for no in activeScore().noteObjs():
        if no.isChord()or no.isRest():
            t = dict(type = 'text', x = 0, y = -3.0,
            font = dict(height = 12, pitchAndFamily = 0, face = 'Times New Roman', weight = 0))
            t['content'] = ' '
            t['tag'] = '2086-51' # keine Bsse, nur Tne umwandeln
            no.addDrawObj(t)
            break
